import { IPosition } from "./Interface.type";

export const calculateOptimalPosition = (
  targetId: string,
  popupWidth: number = 50,
  popupHeight: number = 50,
  offset?: { x?: number; y?: number }
): IPosition | null => {
  const targetElement = document.getElementById(targetId);
  if (!targetElement) return null;

  const targetRect = targetElement.getBoundingClientRect();
  const viewportWidth = window.innerWidth;
  const viewportHeight = window.innerHeight;
  const scrollX = window.scrollX;
  const scrollY = window.scrollY;

  // Set default offset values for better spacing
  const defaultOffset = 5;
  const offsetX = (offset?.x ?? defaultOffset) + 25; // Extra spacing
  const offsetY = (offset?.y ?? defaultOffset) + 25;

  // Determine if the screen size is mobile/tablet
  const isMobile = viewportWidth <= 600;

  let position: IPosition;

  if (isMobile) {
    // Mobile and tablet view logic remains unchanged
    position = {
      top: Math.max(targetRect.top + scrollY + offsetY, 10),
      left: Math.max(targetRect.left + scrollX, 10),
      transformOrigin: 'top left',
    };
  } else {
    // Desktop view logic
    const isRightSide = targetRect.left > viewportWidth / 2;

    // Calculate the vertical position
    let topPosition: number;
    if (targetRect.bottom + popupHeight + offsetY <= viewportHeight + scrollY) {
      // Position below if there's enough space
      topPosition = targetRect.bottom + scrollY + offsetY;
    } else if (targetRect.top - popupHeight - offsetY >= scrollY) {
      // Position above if there's enough space
      topPosition = targetRect.top + scrollY - popupHeight - offsetY;
    } else {
      // Center the popup vertically if it doesn't fit above or below
      topPosition = Math.max(
        scrollY + defaultOffset,
        Math.min(scrollY + (viewportHeight - popupHeight) / 2, scrollY + viewportHeight - popupHeight - defaultOffset)
      );
    }

    // Calculate the horizontal position
    let leftPosition: number;
    if (isRightSide) {
      if (targetRect.left - popupWidth - offsetX >= scrollX) {
        // Position to the left of the target
        leftPosition = targetRect.left + scrollX - popupWidth - offsetX;
      } else {
        // Center horizontally if left doesn't fit
        leftPosition = Math.max(
          scrollX + defaultOffset,
          Math.min(scrollX + (viewportWidth - popupWidth) / 2, scrollX + viewportWidth - popupWidth - defaultOffset)
        );
      }
    } else {
      if (targetRect.right + popupWidth + offsetX <= scrollX + viewportWidth) {
        // Position to the right of the target
        leftPosition = targetRect.right + scrollX + offsetX;
      } else {
        // Center horizontally if right doesn't fit
        leftPosition = Math.max(
          scrollX + defaultOffset,
          Math.min(scrollX + (viewportWidth - popupWidth) / 2, scrollX + viewportWidth - popupWidth - defaultOffset)
        );
      }
    }

    // Set the transform origin based on the calculated position
    let transformOrigin: string;
    if (topPosition < targetRect.top + scrollY) {
      transformOrigin = 'bottom';
    } else if (topPosition > targetRect.bottom + scrollY) {
      transformOrigin = 'top';
    } else {
      transformOrigin = 'center';
    }

    if (leftPosition < targetRect.left + scrollX) {
      transformOrigin += ' right';
    } else if (leftPosition > targetRect.right + scrollX) {
      transformOrigin += ' left';
    } else {
      transformOrigin += ' center';
    }

    position = {
      top: topPosition,
      left: leftPosition,
      transformOrigin,
    };
  }

  return position;
};
