import React, { useEffect, useState } from "react";
import Overlay from "../Overlay/Overlay";
import { calculateOptimalPosition } from "../OptimalPosition/CalculateOptimalPosition";
import "../Overlay/Overlay.css";
import { useViewport } from "../../Hooks/useViewport";
import { IPosition, ITourGuideProps } from "./Interface.type";

export const TourGuide: React.FC<ITourGuideProps> = ({ steps }) => {
  const viewportWidth = useViewport(); // Assuming a custom hook to get viewport width
  const isMobile = viewportWidth <= 600;
  const isTablet = viewportWidth > 600 && viewportWidth <= 1024;

  const [isVisible, setIsVisible] = useState(false);
  const [currentStep, setCurrentStep] = useState(0);
  const [popupPosition, setPopupPosition] = useState<IPosition | null>(null);
  const [colorConfig, setColorConfig] = useState({
    Modal: "#ffffff",
    textColor: "#000000",
    buttonColor: "#007BFF",
  });

  // Start the tour when the website opens and there are steps.
  useEffect(() => {
    if (steps.length) setIsVisible(true);
  }, [steps]);

  // Load custom color configuration from local storage.
  useEffect(() => {
    const savedConfig = localStorage.getItem("colorConfig");
    if (savedConfig) {
      setColorConfig(JSON.parse(savedConfig));
    }
  }, []);

  // Update the popup position based on the current step.
  useEffect(() => {
    const step = steps[currentStep];
    const position = calculateOptimalPosition(step.id, 200, 100); // Reduced popup size
    if (position) {
      setPopupPosition(position);
    }

    // Auto-scroll to the step's section
    const element = document.getElementById(step.id);
    if (element) {
      element.scrollIntoView({ behavior: "smooth", block: "center" });
    }
  }, [currentStep, steps]);

  // Handlers for navigation, closing, and restarting the tour.
  const handleNext = () =>
    setCurrentStep((prev) => Math.min(prev + 1, steps.length - 1));
  const handlePrevious = () => setCurrentStep((prev) => Math.max(prev - 1, 0));
  const handleClose = () => setIsVisible(false);
  const handleSkip = () => setIsVisible(false);
  const handleRestart = () => {
    setCurrentStep(0); // Reset the tour to the first step.
    setIsVisible(true); // Make the tour visible again.
  };

  const handleColorChange = (key: string, value: string) => {
    const updatedConfig = { ...colorConfig, [key]: value };
    setColorConfig(updatedConfig);
    localStorage.setItem("colorConfig", JSON.stringify(updatedConfig));
  };

  const { Modal, textColor, buttonColor } = colorConfig;

  return (
    <>
      {/* Restart button (always visible) */}
      <button
        onClick={handleRestart}
        className="fixed bottom-4 right-4 p-3 bg-blue-500 text-white rounded-lg shadow-lg hover:bg-blue-600"
        style={{
          zIndex: 1000, // Ensure it stays on top of other elements
        }}
      >
        Restart Tour
      </button>

      {isVisible && popupPosition && (
        <>
          <Overlay
            activeElementId={steps[currentStep]?.id}
            onClose={handleClose}
            colorConfig={{
              overlayBackgroundColor: "rgba(0, 0, 0, 0.5)",
              highlightColor: "whitesmoke",
            }}
          />

          <div
            className={`overlay-modal ${
              isMobile || isTablet
                ? "bottom-4 left-1/2 transform -translate-x-1/2"
                : ""
            }`}
            style={{
              top: isMobile || isTablet ? "auto" : popupPosition?.top,
              bottom: isMobile || isTablet ? "20px" : "auto",
              left: isMobile || isTablet ? undefined : popupPosition?.left,
              transformOrigin: popupPosition?.transformOrigin,
              backgroundColor: Modal,
              color: textColor,
              width: isMobile ? "70%" : "300px",
              padding: "0.5rem",
            }}
          >
            {/* Modal content */}
            <button
              className="absolute top-2 right-2 hover:bg-gray-200 transition"
              onClick={handleClose}
            >
              <span className="font-bold text-sm" style={{ color: buttonColor }}>
                &times;
              </span>
            </button>

            <h3 className="text-lg font-semibold text-center mb-2 leading-tight">
              {steps[currentStep]?.title}
            </h3>
            <p className="text-sm text-center mb-4">
              {steps[currentStep]?.description}
            </p>

            {steps[currentStep]?.logoUrl && (
              <div className="flex justify-center mb-4">
                <img
                  src={steps[currentStep]?.logoUrl}
                  alt="Logo"
                  className="w-10 h-10 object-contain"
                />
              </div>
            )}

            <div className="button-container flex justify-between items-center gap-2">
              <button
                onClick={handleSkip}
                className="py-1 px-2 rounded-lg w-full sm:w-auto"
                style={{ backgroundColor: buttonColor, color: textColor }}
              >
                Skip
              </button>
              <button
                onClick={handlePrevious}
                disabled={currentStep === 0}
                className="py-1 px-2 rounded-lg w-full sm:w-auto"
                style={{
                  backgroundColor: buttonColor,
                  color: textColor,
                  opacity: currentStep === 0 ? 0.6 : 1,
                }}
              >
                Previous
              </button>
              <span className="text-center flex-grow w-full sm:w-auto">
                {currentStep + 1} / {steps.length}
              </span>
              <button
                onClick={handleNext}
                disabled={currentStep === steps.length - 1}
                className="py-1 px-2 rounded-lg w-full sm:w-auto"
                style={{
                  backgroundColor: buttonColor,
                  color: textColor,
                  opacity: currentStep === steps.length - 1 ? 0.6 : 1,
                }}
              >
                Next
              </button>
            </div>

            <div>
              <div className="flex items-center justify-start gap-2">
                Customize Colors -
                {["Modal", "textColor", "buttonColor"].map((key) => (
                  <input
                    key={key}
                    type="color"
                    value={(colorConfig as any)[key]}
                    onChange={(e) => handleColorChange(key, e.target.value)}
                    className="w-5 h-5 cursor-pointer"
                    title={key}
                  />
                ))}
              </div>
            </div>
          </div>
        </>
      )}
    </>
  );
};
